#!/bin/sh
#
#  solo.sh -- demo TopoVista, TVX, token service together on a single machine
#
#  usage:    solo.sh [region [tile [maxliteral]]]
#  example:  solo.sh supai 252L6279f 8000

#  configure data sources here
DATA=../../data

#  get command arguments
REGION=${1-supai}
TILE=$2
LITSIZE=${3-8000}

#  set internal variables
TSFIG=solo.cfg

#  enable message tracing by topo
TSVC_TRACE=ON
export TSVC_TRACE

#  pick a starting tile (the fifth in the directory) if none is given
if [ x$TILE = x ]; then
   TILE=`ls $DATA/$REGION | grep '\.tvg$' | sed -n 5p`
   TILE=${TILE%.tvg}
fi

#  sanity check that directory and tile exist
ls -l $DATA/$REGION/$TILE.tvg || exit 1

#  kill background processes after normal or abnormal exit
trap 'X=$?; kill $SERVER $EXPLORER; exit $X' 0 1 2 15

#  trace commands; abort on any error
set -x -e

#  add TopoVista bin directory to command path
export PATH=../../bin:$PATH

#  start server in background, with message tracing
tvjava toksvc.Server -m $TSFIG 0 &
SERVER=$!

#  register initial tile for TopoVista
tvjava toksvc.Broker $TSFIG /dev/stdin <<==EOF==
   request $TILE x
   release $TILE $DATA/$REGION/$TILE.tvg
   close
==EOF==

#  start TVX in background
tvjava explorer.Survey \
   -l $LITSIZE -d $DATA/$REGION -t $TSFIG -u $DATA/$REGION -i 1000000 $TILE &
EXPLORER=$!

#  run TopoVista in foreground
topo -c $TSFIG $TILE
