/*
 * thread.c --
 *
 *	This file implements functions for multithreaded programming:
 *	threads, mutexes, and condition variables.
 *	It is based on swarm/sys/unix/swarm/thread.c and .../mutex.c.
 */

static char rcsid[] =
"$Id: thread.c,v 1.4 2003/10/15 23:55:11 gmt Exp $";

#include "tokInt.h"

#include <pthread.h>

/*
 * TokSpawn(proc, arg) -- start proc(arg) running in another thread.
 */
void
TokSpawn(void *(*threadProc)(void *), void *arg) {
    pthread_t       thread;
    pthread_create(&thread, NULL, threadProc, arg);
    pthread_detach(thread);
}

/*
 * TokInitMutex(m) -- initialize a mutex.
 */
void
TokInitMutex(TokMutex *m) {
    pthread_mutex_init(m, NULL);
}

/*
 * TokLockMutex(m) -- lock a mutex.
 */
void
TokLockMutex(TokMutex *m) {
    pthread_mutex_lock(m);
}

/*
 * TokUnlockMutex(m) -- unlock a mutex.
 */
void
TokUnlockMutex(TokMutex *m) {
    pthread_mutex_unlock(m);
}

/*
 * TokDestroyMutex(m) -- destroy a mutex.
 */
void
TokDestroyMutex(TokMutex *m) {
    pthread_mutex_destroy(m);
}

/*
 * TokInitCond(cv) -- initialize a condition variable.
 */
void
TokInitCond(TokCond *cv) {
    pthread_cond_init(cv, NULL);
}

/*
 * TokWaitCond(cv, m) -- wait for condition variable cv using mutex m.
 */
void
TokWaitCond(TokCond *cv, TokMutex *m) {
    pthread_cond_wait(cv, m);
}

/*
 * TokSignalCond(cv) -- signal a condition variable while holding its mutex.
 */
void
TokSignalCond(TokCond *cv) {
    pthread_cond_broadcast(cv);
}

/*
 * TokDestroyCond(cv) -- destroy a condition variable.
 */
void
TokDestroyCond(TokCond *cv) {
    pthread_cond_destroy(cv);
}
