/*  basic.h -- general macro definitions for all programs  */

#ifndef BASIC_H
#define BASIC_H

#if (defined(WIN32))
   #include <sys/types.h>
   #include <sys/timeb.h>
   #include <fcntl.h>
   #include <io.h>
   #define SET_BINARY_MODE(f) setmode(fileno(f), O_BINARY)
#else
   #include <sys/time.h>
   #include <sys/resource.h>
   #include <unistd.h>
   #define SET_BINARY_MODE(f)
#endif

#include <limits.h>


/*
 * =========== Common Constants =============
 */
#ifndef PI
   #define PI 3.1415926535897932385
#endif

#define ARCSEC_PER_RAD ((360 * 60 * 60) / (2.0 * PI))


/*
 * =========== Common Macros =============
 */
#ifndef MAX
   #define MAX(a, b) (((a)>(b))?(a):(b))
#endif

#ifndef MIN
   #define MIN(a, b) (((a)>(b))?(b):(a))
#endif

#ifndef ABS
   #define ABS(x) (((x)<0)?(-(x)):(x))
#endif

#define LEFT_TURN(x1,y1,x2,y2,x3,y3) \
(((y2)-(y1))*((x3)-(x1))-((x2)-(x1))*((y3)-(y1)) < 0)

#define LEFT_OR_NO_TURN(x1,y1,x2,y2,x3,y3) \
(((y2)-(y1))*((x3)-(x1))-((x2)-(x1))*((y3)-(y1)) <= 0)


/*
 * ============ Common Structures ============
 */

typedef struct { double u, v; } UV;
typedef int Coord2d[2];
typedef float Coord2f[2];
typedef float Coord3f[3];

typedef struct {
    double ua, ub, uc, ud;
    double va, vb, vc, vd;
} Transform;


/*
 * ============ Common prototypes ============
 */

/* trans.c */
Transform *createTransform(UV swS, UV nwS, UV neS, UV seS,
			   UV swT, UV nwT, UV neT, UV seT);
void transform(Transform *f, UV s, UV *t);
void utm2lola(int zone, double x, double y, double *lon, double *lat);
void lola2utm(double lon, double lat, double *x, double *y);
void datum2datum(int datumIn, UV lolaIn, int datumOut, UV *lolaOut);

/* utils.c */
void fatal(char *msg);
void pfatal(char *msg);
void *alloc(size_t nbytes);
void *zalloc(size_t nbytes);
char *salloc(char *s);
void nospace(void);
char *pack4(void *buf, int n);
int unpack4(void *buf);
char *preskip(char *s, char *prefix);
unsigned int tileID(char *s);
double getTime(void);

#endif   /* BASIC_H */
