/*  inhttp.c -- I/O functions for reading from a socket  */

#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include "topo.h"



/*  openHTTP(URL) -- create instruct for reading via HTTP.  */

inp openHTTP(char *URL)
{
  int fd, status;
  char response[MAX_URL_LEN];
  url u;
  inp in;
  struct hostent *hostaddr;
  struct sockaddr_in sockaddr;
  FILE *sockfile;

  if( strlen(URL) > MAX_URL_LEN )
    return NULL;
  if( !parseURL(URL, &u) )
    return NULL;
  if( u.port == 0 )
    u.port = 80;

  /* look up host */
  hostaddr = gethostbyname(u.host);
  if (!hostaddr)
    return NULL;

  /* open socket */
  memset(&sockaddr, 0, sizeof(sockaddr));
  sockaddr.sin_family = AF_INET;
  sockaddr.sin_port = htons(u.port);
  memcpy(&sockaddr.sin_addr, hostaddr->h_addr, hostaddr->h_length);
  if ((fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
    return NULL;
  if (connect(fd,(struct sockaddr *)&sockaddr, sizeof(struct sockaddr)) < 0)
    { close(fd); return NULL; }
  sockfile = fdopen(fd, "r+b");
  SET_BINARY_MODE(sockfile);

  /* send GET request */
  fprintf(sockfile, "GET /%s HTTP/1.0\n\n", u.path);
  fflush(sockfile);

  /* set up instruct and check response code */
  in = openFP(sockfile);
  if (in->rline(in, response, MAX_URL_LEN) == NULL)
    { in->close(in); return NULL; }
  if (sscanf(response, "%*s %d", &status) != 1 || status != 200)
    { in->close(in); return NULL; }

  /* read through end of header */
  while (in->rline(in, response, MAX_URL_LEN) != NULL)
    if (response[0] == '\0')
      return in;

  in->close(in);
  return NULL;
}
