/*  neighbor.c -- find neighbors of triangle  */

#include "topo.h"



/*
 * The following chart describes the neighbor calculations in a
 * recursive fashion: (Note: null concatenated with anything is null)
 *
 * N1(null) = N1(0) = N1(1) = null
 * N1(w0) = N3(w)1
 * N1(w1) = w0
 *
 * N2(null) = N2(0) = N2(1) = null
 * N2(w0) = w1
 * N2(w1) = N3(w)0
 *
 * N3(null) = null
 * N3(0) = 1
 * N3(1) = 0
 * N3(w0) = N2(w)1
 * N3(w1) = N1(w)0
 */



static Triangle cuz3(Triangle t);



/* Returns neighbor opposite vertex 1 of t at same level of tree */

static Triangle cuz1(Triangle t)
{
  if( t.tile == NULL ) return t;
  if( t.i == 1 ) {
    t.tile = t.tile->L1;
  } else if( t.i == 2 ) {
    t.tile = t.tile->R1;
  } else if( t.i & 1 ) {	/* t is a left child */
    t.i = PARENT(t.i);
    t = cuz3(t);
    t.i = RCHILD(t.i);
  } else {			/* t is a right child */
    t.i = t.i - 1;
  }
  return t;
}



/* Returns neighbor opposite vertex 2 of t at same level of tree */

static Triangle cuz2(Triangle t)
{
  if( t.tile == NULL ) return t;
  if( t.i == 1 ) {
    t.tile = t.tile->L2;
  } else if( t.i == 2 ) {
    t.tile = t.tile->R2;
  } else if( t.i & 1 ) {	/* t is a left child */
    t.i = t.i + 1;
  } else {			/* t is a right child */
    t.i = PARENT(t.i);
    t = cuz3(t);
    t.i = LCHILD(t.i);
  }
  return t;
}



/* Returns neighbor opposite vertex 3 of t at same level of tree */

static Triangle cuz3(Triangle t)
{
  if( t.tile == NULL ) return t;
  if( t.i == 1 ) {		/* top NW corner */
    t.i = 2;
  } else if( t.i == 2 ) {	/* top SE corner */
    t.i = 1;
  } else if( t.i & 1 ) {	/* t is a left child */
    t.i = PARENT(t.i);
    t = cuz2(t);
    t.i = RCHILD(t.i);
  } else {			/* t is a right child */
    t.i = PARENT(t.i);
    t = cuz1(t);
    t.i = LCHILD(t.i);
  }
  return t;
}



/* Returns neighbor opposite vertex 1 of t */

Triangle nbr1(Triangle t)
{
  Triangle n;

  n = cuz1(t);
  if( SMALLN1(t) ) {
    n.i = LCHILD(n.i);
  }
  return n;
}



/* Returns neighbor opposite vertex 2 of t */

Triangle nbr2(Triangle t)
{
  Triangle n;

  n = cuz2(t);
  if( SMALLN2(t) ) {
    n.i = RCHILD(n.i);
  }
  return n;
}



/* Returns neighbor opposite vertex 3 of t */

Triangle nbr3(Triangle t)
{
  Triangle n;

  n = cuz3(t);
  if( BIGN3(t) ) {
    n.i = PARENT(n.i);
  }
  return n;
}
