/*  print.c -- print routines */

#include "topo.h"

static double scaleZ;
static FILE *fp;



static void printTri(Triangle t,
		     int x1, int y1, int x2, int y2, int x3, int y3)
{
  double z1, z2, z3;
  double px, py, pz, len;
  double nx, ny, nz;
  int xx, yy;
			
  z1 = EMDATA(t.tile, x1, y1);
  z2 = EMDATA(t.tile, x2, y2);
  z3 = EMDATA(t.tile, x3, y3);

  px = scaleZ * ((y2-y1)*(z3-z1) - (z2-z1)*(y3-y1));
  py = scaleZ * ((z2-z1)*(x3-x1) - (x2-x1)*(z3-z1));
  pz = (double)mPerCell * ((x2-x1)*(y3-y1) - (y2-y1)*(x3-x1));
  len = px * px + py * py + pz * pz;
  len = sqrt(len);
  nx = px / len;
  ny = py / len;
  nz = pz / len;

  xx = t.tile->xx;
  yy = t.tile->yy;
  fprintf(fp, " facet normal %f %f %f\n", nx, ny, nz);
  fprintf(fp, "  outer loop\n");
  fprintf(fp, "   vertex %d %d %f\n", x1*mPerCell + xx, y1*mPerCell + yy,
	  (z1 - ELEVATION_BIAS)*scaleZ);
  fprintf(fp, "   vertex %d %d %f\n", x2*mPerCell + xx, y2*mPerCell + yy,
	  (z2 - ELEVATION_BIAS)*scaleZ);
  fprintf(fp, "   vertex %d %d %f\n", x3*mPerCell + xx, y3*mPerCell + yy,
	  (z3 - ELEVATION_BIAS)*scaleZ);
  fprintf(fp, "  endloop\n");
  fprintf(fp, " endfacet\n");
}



static void printDFS(Triangle t,
		     int x1, int y1, int x2, int y2, int x3, int y3)
{
  int xm, ym;

  if( ONSURFACE(t) ) {
    if( !cullFOV || INFOV(t) ) {
      printTri(t, x1, y1, x2, y2, x3, y3);
    }
  } else if( !ISLEAF(t) ) {
    xm = (x1 + x2) / 2;
    ym = (y1 + y2) / 2;
    t.i = LCHILD(t.i);
    printDFS(t, x3, y3, x1, y1, xm, ym);
    t.i += 1;
    printDFS(t, x2, y2, x3, y3, xm, ym);
  }
}



void printSurface(char *fname)
{
  Tile *tile;
  Triangle t;
  Wrap *wrap;
  WrapGen *wg;

  fp = fopen(fname,"w");
  if( fp == NULL ) {
    fprintf(stderr, "Can't open %s for writing.\n", fname);
    return;
  }
  fprintf(stderr, "Writing triangle list to %s...", fname);

  fprintf(fp, "solid %s created by Topovista\n", fname);
  scaleZ = vexagger / 10.0;

  LOCK(wrapLock);
  wg = genWraps(TYPE_TILE);
  while( (wrap = nextWrap(wg)) != NULL ) {
    tile = wrap->data;
    t.tile = tile;
    t.i = LCHILD(0);
    printDFS(t, tile->lx1,tile->ly1,tile->lx2,tile->ly2,tile->lx3,tile->ly3);
    t.i = RCHILD(0);
    printDFS(t, tile->rx1,tile->ry1,tile->rx2,tile->ry2,tile->rx3,tile->ry3);
  }
  UNLOCK(wrapLock);

  fprintf(fp, "endsolid %s\n", fname);
  fprintf(stderr, "Done\n");
  fclose(fp);
}
