/*  trans.c -- coordinate system transformations  */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "basic.h"



/* transform(f, s, t) -- transform s to create t using transformation f */

void transform(Transform *f, UV s, UV *t)
{
  t->u = f->ua + s.u * f->ub + s.v * f->uc + s.u * s.v * f->ud;
  t->v = f->va + s.u * f->vb + s.v * f->vc + s.u * s.v * f->vd;
}



/* createTransform(swS,nwS,neS,seS,swT,nwT,neT,seT) -- create transformation */

Transform *createTransform(UV swS, UV nwS, UV neS, UV seS,
			   UV swT, UV nwT, UV neT, UV seT)
{
  Transform *f;
  double x1, y1, x2, y2, x3, y3, x4, y4;
  double s1, s2, s3, s4;
  double a11, a12, a13, a14, a21, a22, a23, a24;
  double a31, a32, a33, a34, a41, a42, a43, a44;
  double det;

  f = (Transform *)malloc(sizeof(Transform));
  if (f == NULL) nospace();

  x1 = swS.u;
  y1 = swS.v;
  x2 = nwS.u;
  y2 = nwS.v;
  x3 = neS.u;
  y3 = neS.v;
  x4 = seS.u;
  y4 = seS.v;

  det = (x1 * x2 + x3 * x4) * (y1 - y2) * (y4 - y3)
    + (x1 * x3 + x2 * x4) * (y1 - y3) * (y2 - y4)
    + (x1 * x4 + x2 * x3) * (y1 - y4) * (y3 - y2);

  if (det == 0) {
    fprintf(stderr, "Transform is singular.\n");
    exit(1);
  }

  a31 = x3 * x4 * (y4 - y3) + x2 * x4 * (y2 - y4) + x2 * x3 * (y3 - y2);
  a34 = x2 * x3 * (y2 - y3) + x1 * x3 * (y3 - y1) + x1 * x2 * (y1 - y2);
  a32 = x3 * x4 * (y3 - y4) + x1 * x4 * (y4 - y1) + x1 * x3 * (y1 - y3);
  a11 = x2 * y3 * x4 * y4 - x2 * x3 * y3 * y4 - x3 * y2 * x4 * y4
    + x3 * x2 * y2 * y4 + x4 * y2 * x3 * y3 - x4 * x2 * y2 * y3;
  a44 = x2 * y3 - y2 * x3 - x1 * y3 + y1 * x3 + x1 * y2 - y1 * x2;
  a14 = - x1 * y2 * x3 * y3 + x1 * x2 * y2 * y3 + x2 * y1 * x3 * y3
    - x2 * x1 * y1 * y3 - x3 * y1 * x2 * y2 + x3 * x1 * y1 * y2;
  a33 = x2 * x4 * (y4 - y2) + x1 * x4 * (y1 - y4) + x1 * x2 * (y2 - y1);
  a12 =  - x1 * y3 * x4 * y4 + x1 * x3 * y3 * y4 + x3 * y1 * x4 * y4
    - x3 * x1 * y1 * y4 - x4 * y1 * x3 * y3 + x4 * x1 * y1 * y3;
  a21 = - y3 * x4 * y4 + x3 * y3 * y4 + y2 * x4 * y4
    - x2 * y2 * y4 - y2 * x3 * y3 + x2 * y2 * y3;
  a41 = - x3 * y4 + y3 * x4 + x2 * y4 - y2 * x4 - x2 * y3 + y2 * x3;
  a24 = y2 * x3 * y3 - x2 * y2 * y3 - y1 * x3 * y3
    + x1 * y1 * y3 + y1 * x2 * y2 - x1 * y1 * y2;
  a42 = x3 * y4 - y3 * x4 - x1 * y4 + y1 * x4 + x1 * y3 - y1 * x3;
  a22 = y3 * x4 * y4 - x3 * y3 * y4 - y1 * x4 * y4
    + x1 * y1 * y4 + y1 * x3 * y3 - x1 * y1 * y3;
  a43 = - x2 * y4 + y2 * x4 + x1 * y4 - y1 * x4 - x1 * y2 + y1 * x2;
  a23 = - y2 * x4 * y4 + x2 * y2 * y4 + y1 * x4 * y4
    - x1 * y1 * y4 - y1 * x2 * y2 + x1 * y1 * y2;
  a13 = x1 * y2 * x4 * y4 - x1 * x2 * y2 * y4 - x2 * y1 * x4 * y4
    + x2 * x1 * y1 * y4 + x4 * y1 * x2 * y2 - x4 * x1 * y1 * y2;

  s1 = swT.u;
  s2 = nwT.u;
  s3 = neT.u;
  s4 = seT.u;

  f->ua = (a11 * s1 + a12 * s2 + a13 * s3 + a14 * s4) / det;
  f->ub = (a21 * s1 + a22 * s2 + a23 * s3 + a24 * s4) / det;
  f->uc = (a31 * s1 + a32 * s2 + a33 * s3 + a34 * s4) / det;
  f->ud = (a41 * s1 + a42 * s2 + a43 * s3 + a44 * s4) / det;

  s1 = swT.v;
  s2 = nwT.v;
  s3 = neT.v;
  s4 = seT.v;

  f->va = (a11 * s1 + a12 * s2 + a13 * s3 + a14 * s4) / det;
  f->vb = (a21 * s1 + a22 * s2 + a23 * s3 + a24 * s4) / det;
  f->vc = (a31 * s1 + a32 * s2 + a33 * s3 + a34 * s4) / det;
  f->vd = (a41 * s1 + a42 * s2 + a43 * s3 + a44 * s4) / det;

  return f;
}



/*
  Standard Molodensky Datum Transformation
  See NIMA TR 8350.2

  ************************ WARNING ******************************************
  NIMA TR 8350.2 contains Molodensky parameters to convert
  from an arbitrary datum to WGS84.  To convert from datum A to datum B,
  I use molodensky(Ax-Bx,Ay-By,Az-Bz,Aa,Af,Ba,Bf) where Ax,Ay,Az are the
  shift to convert A to WGS84; Bx,By,Bz are the shift to convert B to WGS84;
  Aa,Af,Ba,Bf are the ellipsoid parameters. This may cause errors.
  ************************ WARNING ******************************************
  */

struct datumTable {
  double a;		/* semi-major axis of datum ellipsoid */
  double f;		/* flattening of datum ellipsoid */
  double dx,dy,dz;	/* deltas to transform datum(x,y,z) to WGS84(x,y,z) */
} datum[] = {
  { 6378206.4, 1. / 294.9786982,  -8,  160,  176 }, /* default */
  { 6378206.4, 1. / 294.9786982,  -8,  160,  176 }, /* NAD27 */
  { 6378135.0, 1. / 298.26,        0,    0,    0 }, /* WGS72 */
  { 6378137.0, 1. / 298.257223563, 0,    0,    0 }, /* WGS84 */
  { 6378206.4, 1. / 294.9786982,  61, -285, -181 }, /* OLD_HAWAII_DATUM */
  { 6378206.4, 1. / 294.9786982,  11,   72, -101 }, /* PUERTO_RICO_DATUM */
};

#define NUMBER_OF_DATUMS (sizeof(datum) / sizeof(datum[0]))

void datum2datum(int datumIn, UV lolaIn, int datumOut, UV *lolaOut)
{
  struct datumTable *in, *out;
  double dx,dy,dz,da,df,e2,bbya,lam,slam,clam,phi,sphi,cphi,Rm,Rn,dlam,dphi;

  if( datumIn < 0 || datumIn > NUMBER_OF_DATUMS ) datumIn = 0;
  if( datumOut < 0 || datumOut > NUMBER_OF_DATUMS ) datumOut = 0;
  in = &(datum[datumIn]);
  out = &(datum[datumOut]);

  dx = in->dx - out->dx;
  dy = in->dy - out->dy;
  dz = in->dz - out->dz;

  da = out->a - in->a;
  df = out->f - in->f;
  e2 = in->f * (2. - in->f);
  bbya = 1. - in->f;
  lam = lolaIn.u / ARCSEC_PER_RAD;
  slam = sin(lam);
  clam = cos(lam);
  phi = lolaIn.v / ARCSEC_PER_RAD;
  sphi = sin(phi);
  cphi = cos(phi);
  Rm = in->a * (1 - e2) /
    sqrt((1 - e2 * sphi * sphi)*(1 - e2 * sphi * sphi)*(1 - e2 * sphi * sphi));
  Rn = in->a / sqrt(1 - e2 * sphi * sphi);
  dlam = (-dx * slam + dy * clam) / (Rn * cphi);
  dphi = (-dx * sphi * clam - dy * sphi * slam + dz * cphi +
	  da * (Rn * e2 * sphi * cphi) / in->a +
	  df * (Rm / bbya + Rn * bbya) * sphi * cphi) / Rm;
  lolaOut->u = (lam + dlam) * ARCSEC_PER_RAD;
  lolaOut->v = (phi + dphi) * ARCSEC_PER_RAD;
}



/* This assumes the Clark 1866 ellipsoid (NAD27).  See
 * Map Projections -- A Working Manual  by John P. Snyder
 * U.S. Geological Survey Professional Paper 1395
 */

#define k0 0.9996
#define M0 0.0

void utm2lola(int zone, double x, double y, double *lon, double *lat)
{
  double a = 6378206.4;
  double es = 0.00676866;
  double lam0, M, mu, e1, phi1, eps, phi, lam, C1, T1, N1, R1, D;

  lam0 = ((double)zone - 0.5) * PI / 30.0 - PI;

  /*  x = x - 500000.;*/

  M = y / k0;		/* M0 = 0 */
  mu = M / (a * (1 - es / 4. - 3. * es * es / 64. - 5. * es*es*es / 256.));
  e1 = (1 - sqrt(1 - es)) / (1 + sqrt(1 - es));


  phi1 = mu + (3. * e1 / 2. - 27. * e1 * e1 * e1 / 32.) * sin(2. * mu)
    + (21. * e1 * e1 / 16. - 55. * e1*e1*e1*e1 / 32.) * sin(4. * mu)
    + (151. * e1 * e1 * e1 / 96.) * sin(6. * mu)
    + (1097. * e1 * e1 * e1 * e1 / 512.) * sin(8. * mu);

  eps = es / (1 - es);
  C1 = eps * cos(phi1) * cos(phi1);
  T1 = tan(phi1) * tan(phi1);
  N1 = a / sqrt(1 - es * sin(phi1) * sin(phi1));
  R1 = a*(1-es) / (sqrt(1-es*sin(phi1)*sin(phi1))*(1-es*sin(phi1)*sin(phi1)));
  D = x / (N1 * k0);

  phi = phi1 - (N1 * tan(phi1) / R1) *
    (D*D/2. - (5. + 3.*T1 + 10.*C1 - 4.*C1*C1 - 9.*eps) * D*D*D*D/24. +
     (61. + 90. * T1 + 298. * C1 + 45. * T1*T1 - 252. * eps - 3. * C1*C1) *
     D*D*D*D*D*D/720.);

  lam = lam0 + (D - (1 + 2 * T1 + C1) * D*D*D / 6. +
		(5. - 2. * C1 + 28. * T1 - 3. * C1*C1 +
		 8. * eps + 24. * T1*T1) * D*D*D*D*D / 120.) / cos(phi1);


  *lon = lam * ARCSEC_PER_RAD;
  *lat = phi * ARCSEC_PER_RAD;
}



void lola2utm(double lon, double lat, double *x, double *y)
{
  int zone;
  double a = 6378206.4;
  double es = 0.00676866;
  double lam, phi, lam0, eps, N, T, C, A, M;

  zone = (185 + (int)(lon / 3600.)) / 6;
  lam = lon / ARCSEC_PER_RAD;
  phi = lat / ARCSEC_PER_RAD;
  lam0 = ((double)zone - 0.5) * PI / 30.0 - PI;

  eps = es / (1 - es);
  N = a / sqrt(1 - es * sin(phi) * sin(phi));
  T = tan(phi) * tan(phi);
  C = eps * cos(phi) * cos(phi);
  A = (lam-lam0)*cos(phi);
  M = a*((1.-es/4.-3.*es*es/64.-5.*es*es*es/256.)*phi-
	 (3.*es/8.+3.*es*es/32.+45.*es*es*es/1024.)*sin(2.*phi)+
	 (15.*es*es/256.+45.*es*es*es/1024.)*sin(4.*phi)-
	 (35.*es*es*es/3072.)*sin(6.*phi));
  *x = k0*N*(A+(1-T+C)*A*A*A/6.+(5.-18.*T+T*T+72.*C-58.*eps)*
	     A*A*A*A*A/120.); /* + 500000.; */
  *y = k0*(M-M0+N*tan(phi)*(A*A/2.+(5.-T+9.*C+4.*C*C)*
			    A*A*A*A/24.+(61.-58.*T+T*T+600.*C-330.*eps)*
			    A*A*A*A*A*A/720.));
}
