/*  tvname.c -- utility for translating lat/lon to TopoVista tile names
 *
 *  usage:  tvname nLatitude wLongitude
 *
 *  tvname prints the tile names (for five tile sizes and two different
 *  resolutions) that correspond to the given geographic coordinates.
 *
 *  Latitude and longitude are given as real numbers,
 *  in degrees N and W respectively.
 */

#include "topo.h"

int main(int argc, char *argv[]) {
  double lat, lon, utmx, utmy;
  int k, zone, tileSizeM;
  int spacing = 10;
  char buf[20];

  if( argc != 3
  || sscanf(argv[1], "%lf", &lat) != 1 || lat < 0 || lat > 84
  || sscanf(argv[2], "%lf", &lon) != 1 || lon < -180 || lon > 180) {
    fprintf(stderr, "usage: tvname nLatitude wLongitude\n");
    exit(1);
  }

  if( lon > 0 )
    lon = -lon;
  zone = ((int)(186 + lon)) / 6;
  lola2utm(3600 * lon, 3600 * lat, &utmx, &utmy);

  for( spacing = 10; spacing <= 30; spacing += 20 ) {
    printf("(%dm) ", spacing);
    for( k = 6; k <= 10; k++ ) {
      tileSizeM = (1 << k) * spacing;
      sprintf(buf, "%d%c%d%c.tvg",
        UTM2XXX(utmx), zone + 'A' - 1, UTM2YYY(utmy), k + 'a' - 1);
      printf("%14s", buf);
    }
    printf("\n");
  }

  return 0;
}
