/*
 * ptest.c --
 *
 *	This file implements a test of the functions in pickle.c.
 */

static char rcsid[] =
"$Id: ptest.c,v 1.6 2001/07/03 22:08:30 gmt Exp $";

#include "tokInt.h"

static long vals[] = {
    0, 1, 2, 3, 4, 5, 6, 7, 11, 13, 17, 19, 23, 29, 30, 31, 32, 33, 34, 37,
    41, 43, 47, 62, 63, 64, 65, 66, 67, 91, 127, 128, 129, 130, 131, 173,
    229, 254, 255, 256, 257,  271, 331, 383, 419, 491, 511, 512, 513, 777,
    0x0345, 0x03FF, 0x0400, 0x0401,
    0x0567, 0x07FF, 0x0800, 0x0801,
    0x0999, 0x0FFF, 0x1000, 0x1001,
    0x1234, 0x1FFF, 0x2000, 0x2001,
    0x3579, 0x3FFF, 0x4000, 0x4001,
    0x6789, 0x7FFF, 0x8000, 0x8001,
    0xBACE, 0xFFFF, 0x10000, 0x10001,
    0x123456, 0xFFFFFF, 0x1000000, 0x1000001,
    0x12345678L, 0x7FFFFFFFL,
};

static char *strings[] = {
    "", "1", "23", "abc", "four", "50010", "walla walla",
};

static char *bytes[] = {
    "", "one", "two", "three", "four", "five", "six", "seven", "eight",
};

int
main(int argc, char *argv[]) {
    int i, n;
    long v1, v2;
    int ns = sizeof(strings) / sizeof(strings[0]);
    int nv = sizeof(vals) / sizeof(vals[0]);
    int nb = sizeof(bytes) / sizeof(bytes[0]);
    TokPickle p = TokEmptyPickle();
    char buf[10];
    char *s;

    for (i = 0; i < nv; i++) {
	TokPickleInt(p, vals[i]);
	TokPickleInt(p, -vals[i]);
    }
    for (i = 0; i < ns; i++) {
	TokPickleString(p, strings[i]);
    }
    for (i = 0; i < nb; i++) {
	TokPickleBytes(p, bytes[i], strlen(bytes[i]));
    }

    printf("Pickled size: %d\n", TokPickleSize(p));

    for (i = 0; i < nv; i++) {
	v1 = TokUnpickleInt(p);
	v2 = TokUnpickleInt(p);
	printf("%10ld %10ld %10ld\n", vals[i], v1, v2);
	if (v1 != vals[i] || v2 != -vals[i])
	    printf("   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n");
    }
    printf("\n");
    for (i = 0; i < ns; i++) {
	s = TokUnpickleString(p);
	printf("%5s  %s\n", strcmp(s, strings[i]) == 0 ? "ok" : "ERROR", s);
	TokFree(s);
    }
    printf("\n");
    for (i = 0; i < nb; i++) {		/* test arrays of bytes */
	strcpy(buf, "........");	/* initialize buffer */
	n = TokUnpickleBytes(p, buf, 4);/* deliberately insufficient size */
	printf("%d  %s\n", n, buf);	/* print results */
	if (n != strlen(bytes[i]))
	    printf("^^\n");
    }

    return 0;
}
