/*
 * utest.c --
 *
 *	This file implements a test of some of the user-callable functions.
 *
 * usage:  utest configfile
 */

static char rcsid[] =
"$Id: utest.c,v 1.6 2003/10/16 22:17:51 gmt Exp $";

#include "tok.h"

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

void callback(Tok_Token, void *);


int
main(int argc, char *argv[]) {
    Tok_Service svc;
    Tok_Token t;
    char **fig;
    char buf[10];

    if (argc < 2)
	{ fprintf(stderr, "usage: %s configfile\n", argv[0]); exit(1); }
    fig = Tok_Config(argv[1]);
    if (!fig)
	{ fprintf(stderr, "can't load config\n"); exit(1); }
    svc = Tok_Open(fig);
    if (!svc)
	{ fprintf(stderr, "can't open token service\n"); exit(1); }

    t = Tok_Request(svc, "utest", TOK_EXCLUSIVE, callback, "xyzzy");
    if (!t)
	{ fprintf(stderr, "can't get token\n"); exit(1); }
    printf("TOKEN <%s> DATA <%s>\n", Tok_GetName(t), (char *) Tok_GetData(t));
    strcpy(buf, "TestValue");
    Tok_SetData(t, buf, strlen(buf) + 1);
    printf("TOKEN <%s> DATA <%s>\n", Tok_GetName(t), (char *) Tok_GetData(t));
    Tok_Update(t);
    printf("sleeping\n");
    sleep(1);
    Tok_Release(t);
    Tok_Close(svc);
    printf("done\n");
    return 0;
}

void
callback(Tok_Token t, void *arg) {
    printf("CALLBACK <%s> for <%s>\n", (char *) arg, Tok_GetName(t));
}
