package topovista.explorer;

import java.util.*;

import topovista.grid.*;



/**
 *  A Cell object represents the state of one grid cell.
 *
 *  <P> A Cell has a name of the form "xxxUyyyc" where U is an upper case
 *  letter, c is a lower case letter, and xxx and yyy are decimal integers
 *  giving the cell coordinates.
 *  These components set the zone, x, y, k fields when a Cell is constructed.
 */

class Cell {

    public final String name;	// cell name		e.g. "505L467h"
    public final int zone;	// UTM zone		'A' is zone 1
    public final int x, y;	// grid coordinates	e.g. 505, 467
    public final int k;		// cell order		'a' is 1

    public boolean hasdata;	// is data available?
    public boolean visited;	// has cell been "visited"?



public Cell(String name) {
    try {
	int vals[] = TileGrid.parseName(name);
	zone = vals[0];
	x = vals[1];
	y = vals[2];
	k = vals[3];
    } catch (Throwable t) {
	throw new Error("malformed cellname: " + name);
    }
    this.name = name;
}



/**
 *  Returns a string representation of this Cell.
 */
public String toString() {
    StringBuffer b = new StringBuffer("Cell(");
    b.append(name).append(',');
    if (hasdata)  { b.append('d'); }
    if (visited)  { b.append('v'); }
    b.append(')');
    return b.toString();
}



} // class Cell
