package topovista.explorer;

import topovista.toksvc.*;



/**
 *  Explorer is the abstract superclass of agents that
 *  traverse the world using various algorithms.
 */

abstract class Explorer extends Thread {

    final String label;		// label for this explorer
    final char ch;		// selection character for -px command argument



/**
 *  Constructs an Explorer with a given selection character and label.
 */
Explorer(char ch, String label) {
    this.ch = ch;
    this.label = label;
}



/**
 *  Reinitializes an explorer object.
 *  Subclasses may override this to do something useful.
 */
public void init() {
}



/**
 *  Produces the next location in sequence.
 *  Must be implemented by subclasses.
 */
abstract public Cell nextLoc(World w, Cell curr);



/**
 *  Randomly reorders an arbitrary array.
 *  (Used in several subclasses.)
 */
static Object[] shuffle(Object[] a) {
    int n = a.length;
    for (int i = 1; i < n; i++) {
	int j = (int) ((i + 1) * Math.random());
	Object o = a[i];
	a[i] = a[j];
	a[j] = o;
    }
    return a;
}



} // class Explorer
