package topovista.explorer;

/**
 *  Implements a random walk with a slight constant bias towards the
 *  start cell.  The physical equivalent is a shallow cone with the
 *  start at the center.
 */

class WeightedRandom extends Explorer {

    private int x0, y0;

private final static double WEIGHT = 0.10;	// bias weighting (0 to 1)



WeightedRandom() {
    super('w', "weighted random");
}



public void init() {
    x0 = y0 = 0;
}



public Cell nextLoc(World w, Cell curr) {

    int x = curr.x;
    int y = curr.y;

    if (y0 == 0) {
	x0 = curr.x;
	y0 = curr.y;
    }

    double theta = Math.atan2(y0 - curr.y, x0 - curr.x);
    double xbias = 0.25 * WEIGHT * Math.cos(theta);
    double ybias = 0.25 * WEIGHT * Math.sin(theta);

    double r = Math.random();
    if (r < 0.25 + xbias)		{ x++; }
    else if (r < 0.50)			{ x--; }
    else if (r < 0.75 + ybias)		{ y++; }
    else				{ y--; }

    return w.find(curr.zone, x, y, curr.k);
}



} // class WeightedRandom
