package topovista.grid;

import java.io.*;
import java.util.zip.*;

import topovista.util.*;



/**
 *  The BinaryGrid class reads and writes elevation grids using our
 *  own private format.
 */

class BinaryGrid extends Grid {



/**
 *  Constructs a BinaryGrid by reading from a file.
 */
BinaryGrid(TypedInputStream ins, boolean partial) throws IOException {

    ins.readString();			// magic
    cols = ins.readInt();
    rows = ins.readInt();
    header = ins.readString();
    freeform = ins.readString();
    refsys = ins.readInt();
    zone = ins.readInt();
    xyUnit = ins.readInt();
    zUnit = ins.readInt();
    datum = ins.readInt();

    xRes = ins.readDouble();
    xMin = ins.readDouble();
    xMax = ins.readDouble();
    yRes = ins.readDouble();
    yMin = ins.readDouble();
    yMax = ins.readDouble();
    zRes = ins.readDouble();
    zMin = ins.readDouble();
    zMax = ins.readDouble();
    xLL = ins.readDouble();
    yLL = ins.readDouble();

    if (partial) {
	return;
    }

    alloc();
    for (int i = rows - 1; i >= 0; i--) {
	for (int j = 0; j < cols; j++) {
	    elev[i][j] = (char) (ins.readInt() + ELEVATION_BIAS);
	}
    }
}



/**
 *  Writes a Grid object in binary format.
 */
static void write(Grid g) throws IOException {

    GZIPOutputStream gzo = new GZIPOutputStream(System.out);
    TypedOutputStream o = new TypedOutputStream(gzo);

    o.write("BDEM");	// magic identifier (but note that strlen outputs first)
    o.write(g.cols);
    o.write(g.rows);
    o.write(g.header);
    o.write(g.freeform);
    o.write(g.refsys);
    o.write(g.zone);
    o.write(g.xyUnit);
    o.write(g.zUnit);
    o.write(g.datum);

    o.write(g.xRes);
    o.write(g.xMin);
    o.write(g.xMax);
    o.write(g.yRes);
    o.write(g.yMin);
    o.write(g.yMax);
    o.write(g.zRes);
    o.write(g.zMin);
    o.write(g.zMax);
    o.write(g.xLL);
    o.write(g.yLL);

    for (int i = g.rows - 1; i >= 0; i--) {
	g.loadRow(i);
	for (int j = 0; j < g.cols; j++) {
	    o.write(g.elev[i][j] - ELEVATION_BIAS);
	}
	g.discardRow(i);
    }
    o.flush();
    gzo.finish();
}



} // class BinaryGrid
