package topovista.rpserver;

import java.net.*;
import java.util.*;



/**
 *  The listener thread receives messages on a port and enters them in
 *  separate request queues (distinguished by priority) for processing.
 *
 *  The listener is a Thread object that must be explicitly started by
 *  the code that creates it.
 */

class Listener extends Thread implements TVRP {



    private DatagramSocket udp;		// incoming port

    private LinkedList[] rqqueues;	// queues for storing requests



/**
 *  Constructs a listener that receives from the specified port and
 *  enters messages in the given array of queues.
 */
Listener(LinkedList[] rqqueues, int port) throws Exception {
    this.rqqueues = rqqueues;
    udp = new DatagramSocket(port);
}



/**
 *  Loops forever, reading and processing incoming datagrams.
 */
public void run() {

    for (;;) try {
	byte[] buf = new byte[REQ_MAX];
	DatagramPacket in = new DatagramPacket(buf, buf.length);
	udp.receive(in);
	if (!(new String(buf, 0, 4)).equals("TVRP")) {
	    continue;
	}
	Message m = new Message(in);
	m.client.latest = m;
	synchronized (rqqueues) {
	    for (int i = 0; i < m.rqlist.length; i++) {
		Request r = m.rqlist[i];
		rqqueues[r.pri].add(r);
	    }
	    rqqueues.notifyAll();
	}
    } catch (Throwable e) {
	e.printStackTrace();
	System.exit(1);
    }
}



/**
 *  Returns a string representation of this listener.
 */
public String toString() {
    return "Listener(" + rqqueues + "," + udp.getLocalPort() + ")";
}



} // class Listener
