package topovista.rpserver;

import java.net.*;


/**
 *  Represents an incoming message containing multiple discrete requests.
 */
class Message implements TVRP {

    long timestamp;		// arrival timestamp
    Client client;		// sending client
    Request[] rqlist;		// list of individual requests



/**
 *  Constructs a message object from the incoming datagram.
 */
Message(DatagramPacket p) {
    timestamp = System.currentTimeMillis();
    client = Client.lookup(p.getSocketAddress());
    byte[] data = p.getData();
    int n = p.getLength() / 8 - 1;
    rqlist = new Request[n];
    for (int i = 0; i < n; i++) {
	rqlist[i] = new Request(this, data, 8 * (i + 1));
    }
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return "Message(" + client.addr +","+ timestamp +","+ rqlist.length + ")";
}



} // class Message
