package topovista.rpserver;

import java.io.*;
import java.net.*;
import java.util.*;

import topovista.grid.*;



/**
 *  Implements a utility program for fetching a TopoVista tile via TVRP.
 */
public class RPFetch implements TVRP {



/**
 *  Contacts an RPServer on the local host and reads a TopoVista tile.
 *  Does not fully comply with the TVRP protocol spec;
 *  designed for testing this particular server on a perfect network.
 *  Quits after the first "Unavailable" response, so it doesn't
 *  get any error records for tiles smaller than 256x256.
 *
 *  <P> Usage: java RPFetch portnum tilename
 */
public static void main(String[] args) throws Exception {
    int port = 0;
    String tilename = "";
    try {
	port = Integer.parseInt(args[0]);
	tilename = args[1];
    } catch (Exception e) {
	System.err.println("usage: java RPFetch portnum tilename");
	System.exit(1);
    }
    int tid = TileGrid.makeID(tilename);
    String hexID = TileGrid.hexID(tid);

    DatagramSocket udp = new DatagramSocket();
    InetAddress addr = udp.getLocalAddress();

    for (int i = 0; i < RECS_BITS; i++) {

	byte[] req = new byte[16];
	int b = (63 << PRI_SHIFT) | (1 << (RECS_BITS - i - 1));
	Responder.pack(req, 0, 0x54565250);	//  T  V  R  P
	Responder.pack(req, 4, 0);		// 00 00 00 00
	Responder.pack(req, 8, tid);		// tile ID
	Responder.pack(req, 12, b);		// one bit for one record
	udp.send(new DatagramPacket(req, req.length, addr, port));

	byte[] buf = new byte[65536];
	DatagramPacket in = new DatagramPacket(buf, buf.length);
	udp.receive(in);
	if (in.getLength() > INFO_HEADER) {
	    System.out.write(buf, INFO_HEADER, in.getLength() - INFO_HEADER);
	    System.err.println("   " + TileGrid.hexID(tid) + " " +
	        TileGrid.hexID(b) + " " + in.getLength());
	} else {
	    break;
	}
    }
}



} // class RPFetch
