package topovista.rpserver;

import java.util.*;

import topovista.grid.*;
import topovista.util.*;



/**
 *  Implements a TopoVista tile server using TVRP protocol.
 */
public class RPServer implements TVRP {



/**
 *  Runs until halted, accepting and replying to TVRP requests.
 *  The optional command argument is the number of the port to listen on.
 *
 *  <P> Data is read from "*.tvg" files located in and below the current
 *  directory.  File names are assumed to accurately reflect contents
 *  in accordance with TopoVista naming conventions.  Directories are
 *  scanned at startup; additional files cannot be added dynamically.
 */
public static void main(String[] args) throws Exception {

    int port = DEF_PORT;
    int delay = 0;
    double dfract = 0.0;

    String cmdname = "java RPServer";
    Options o = new Options(new String[] {
	"-d pct 0",	"randomly drop a percentage of output messages",
	"-s msec 0",	"sleep msec milliseconds after each packet send",
	"portnum",	"port number",
    }, cmdname, args);

    try {
	delay = Integer.parseInt(o.getValue('s'));
	dfract = Integer.parseInt(o.getValue('d')) / 100.0f;
	int i = o.getIndex();
	port = (args.length > i) ? Integer.parseInt(args[i]) : DEF_PORT;
    } catch (Exception e) {
	o.usage(System.err, cmdname);
	System.exit(1);
    }

    LinkedList[] rqqueues = new LinkedList[PRI_MAX + 1];
    for (int i = 0; i <= PRI_MAX; i++) {
	rqqueues[i] = new LinkedList();
    }

    new Listener(rqqueues, port).start();
    new Responder(rqqueues, delay, dfract).start();
    new Cleaner(rqqueues).start();
    System.err.println("Ready.");
}



} // class RPServer
