//  ConfigMessage.java -- inform client of server configuration


package topovista.toksvc;

import java.net.*;



/**
 *  A ConfigMessage informs a client of the server configuration,
 *  and also tells it its session ID (in the "to" field), if it doesn't already
 *  know that.
 */
class ConfigMessage extends Message {

    int leader;		// index of leader among token servers
    byte[] sstates;	// server state array



final static int MTYPE = 12;	// type code for this Message class



/**
 *  Constructor for synthesizing a new message.
 */
ConfigMessage(int leader, byte[] sstates) {
    super(MTYPE);
    this.leader = leader;
    this.sstates = sstates;
}



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(leader);
    p.put(sstates);
    return p;
}



/**
 *  Constructs a ConfigMessage by reading from a Pickle.
 */
ConfigMessage(Pickle p) {
    super(p);
    leader = p.getInt();
    sstates = p.getByteArray();
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() + '(' + leader + ',' + simage(sstates) + ')';
}



/**
 *  Returns a string that concisely represents a server state array:
 *  The characters "-=+?" respectively represent servers shown as
 *  DOWN, BOOTING, READY, and with illegal states.
 */
public static String simage(byte[] sstates) {
    StringBuffer b = new StringBuffer(sstates.length);
    for (int i = 0; i < sstates.length; i++) {
	switch (sstates[i]) {
	    case Server.DOWN:		b.append('-');	break;
	    case Server.BOOTING:	b.append('=');	break;
	    case Server.READY:		b.append('+');	break;
	    default:			b.append('?');  break;
	}
    }
    return b.toString();
}



}  // class ConfigMessage
