//  CouncilMessage.java -- a message for Council administration.


package topovista.toksvc;

import java.net.*;



/**
 *  A CouncilMessage is the superclass of messages
 *  used for administering a Council.
 */
abstract class CouncilMessage extends Message {

    int term;		// council term (e.g. the 12th administration)
    long msgnum;	// sequential message number



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(term);
    p.put(msgnum);
    return p;
}



/**
 *  Constructs a CouncilMessage by reading from a Pickle.
 */
CouncilMessage(Pickle p) {
    super(p);
    term = p.getInt();
    msgnum = p.getLong();
}



/**
 *  Constructor for synthesizing a new message.
 */
CouncilMessage(int type, int term, long msgnum) {
    super(type);
    this.term = term;
    this.msgnum = msgnum;
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() + "(" + term + "," + msgnum + ")";
}



}  // class CouncilMessage
