//  GrantMessage.java -- assigning a token to a client


package topovista.toksvc;

import java.net.*;



/**
 *  A GrantMessage assigns a requested token to a client.
 */
class GrantMessage extends Message {

    long msgnum;		// number of message being granted
    Tk token;			// the token being assigned



final static int MTYPE = 22;	// type code for this message class



/**
 *  Constructor for synthesizing a new message.
 */
GrantMessage(long msgnum, Tk token) {
    super(MTYPE);
    this.msgnum = msgnum;
    this.token = token;
}



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(msgnum);
    token.pickle(p);
    return p;
}



/**
 *  Constructs a GrantMessage by reading from a Pickle.
 *  A client Token is constructed for the token field.
 */
GrantMessage(Pickle p) {
    super(p);
    msgnum = p.getInt();
    token = new Token(p);
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() + "(" + msgnum + "," +
	token.name + "," + token.dimage() + ")";
}



}  // class GrantMessage
