//  LoginMessage.java -- new client registration


package topovista.toksvc;

import java.net.*;



/**
 *  A LoginMessage from a client requests the return of a ConfigMessage
 *  with a session ID.  The sender supplies a ":port" string as a return
 *  address.  (Any host specification before the colon is ignored.)
 */
class LoginMessage extends Message {

    String address;		// datagram socket: "nn.nn.nn.nn:ppp"



final static int MTYPE = 11;	// type code for this message class



/**
 *  Constructor for synthesizing a new message.
 */
LoginMessage(String address) {
    super(MTYPE);
    this.address = address;
}



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(address);
    return p;
}



/**
 *  Constructs a LoginMessage by reading from a Pickle.
 */
LoginMessage(Pickle p) {
    super(p);
    address = p.getString();
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    String s = super.toString() + "(" + address + ")";
    if (pkt != null) {
	s += "[" + pkt.getAddress().getHostAddress() + "]";
    }
    return s;
}



}  // class LoginMessage
