//  NumberedMessage.java -- a message carrying a Token and a msgnum.


package topovista.toksvc;

import java.net.*;



/**
 *  A NumberedMessage contains a message number, for acknowledgment,
 *  and a token.
 */
abstract class NumberedMessage extends Message {

    long msgnum;		// message number
    Tk token;			// token value



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(msgnum);
    token.pickle(p);
    return p;
}



/**
 *  Constructs a NumberedMessage by reading from a Pickle.
 *  The constructed token is a ServerToken.
 */
NumberedMessage(Pickle p) {
    super(p);
    msgnum = p.getLong();
    token = new ServerToken(p);
}



/**
 *  Constructor for synthesizing a new message.
 */
NumberedMessage(int type, long msgnum, Tk token) {
    super(type);
    this.msgnum = msgnum;
    this.token = token;
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() + "(" + msgnum + "," +
	token.name + "," + token.dimage() + ")";
}



}  // class NumberedMessage
