//  ServerMessage.java -- a server reports its state to the leader


package topovista.toksvc;

import java.net.*;



/**
 *  A ServerMessage reports the state of a token server
 *  to the leader and serves as the token server's heartbeat.
 */
class ServerMessage extends Message {

    int state;		// server state (DOWN, BOOTING, or READY)



final static int MTYPE = 6;	// type code for this message class



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(state);
    return p;
}



/**
 *  Constructs a ServerMessage by reading from a Pickle.
 */
ServerMessage(Pickle p) {
    super(p);
    state = p.getInt();
}



/**
 *  Constructor for synthesizing a new message.
 */
ServerMessage(int state) {
    super(MTYPE);
    this.state = (byte) state;
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    char c;
    switch (state) {
	case Server.DOWN:	c = '-';  break;
	case Server.BOOTING:	c = '=';  break;
	case Server.READY:	c = '+';  break;
	default:		c = '?';  break;
    }
    return super.toString() + "(" + c + ")";
}



}  // class ServerMessage
