//  TransferMessage.java -- transferring tokens and responsibility


package topovista.toksvc;

import java.net.*;



/**
 *  A TransferMessage transfers tokens and responsibility
 *  from a backup server to a newly started server.
 */
class TransferMessage extends Message {

    /**
     *  The Tokens transferred by this message.
     */
    ServerToken[] tokens;



final static int MTYPE = 7;	// type code for this message class



/**
 *  Constructs a new TransferMessage.
 *  The argument array is not copied.
 */
TransferMessage(ServerToken[] tokens) {
    super(MTYPE);
    this.tokens = tokens;
}



/**
 *  Adds the encoding of this object to a pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(tokens.length);
    for (int i = 0; i < tokens.length; i++) {
	tokens[i].pickle(p);
    }
    return p;
}



/**
 *  Constructs a TransferMessage by reading from a Pickle.
 */
TransferMessage(Pickle p) {
    super(p);
    int n = p.getInt();
    tokens = new ServerToken[n];
    for (int i = 0; i < n; i++) {
	tokens[i] = new ServerToken(p);
    }
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() + "[" + tokens.length + "]";
}



}  // class TransferMessage
