package topovista.tvserver;

import java.io.*;
import java.net.*;
import java.util.*;

import topovista.grid.*;



/**
 *  Implements a utility program for fetching a TopoVista tile via TVTP.
 */
public class TVFetch {



private static int TVHEADER = 14;

private static int MAXRESPONSE = 8192;



/**
 *  Contacts a TVServer on the local host and reads a TopoVista tile.
 *
 *  <BR> Usage: java TVFetch portnum tilepath
 */
public static void main(String[] args) throws Exception {
    int port = 0;
    String path = "";
    try {
	port = Integer.parseInt(args[0]);
	path = args[1];
    } catch (Exception e) {
	System.err.println("usage: java TVFetch portnum path");
	System.exit(1);
    }
    int id = TileGrid.makeID(path);
    String hexID = TileGrid.hexID(id);
    DatagramSocket udp = new DatagramSocket();
    byte[] req = ("GET " + hexID + " " + path).getBytes();
    udp.send(new DatagramPacket(req, req.length, udp.getLocalAddress(), port));
    for (;;) {
	byte[] buf = new byte[MAXRESPONSE];
	DatagramPacket in = new DatagramPacket(buf, buf.length);
	udp.receive(in);
	System.err.println(" " + buf[5] + "/" + buf[6] + "/" + buf[7]);
	if (buf[6] == 63) {		// if EOF
	    break;
	}
	System.out.write(buf, TVHEADER, in.getLength() - TVHEADER);
    }
}



} // class TVFetch
