package topovista.tvsucker;

import java.io.*;



/**
 *  Fetches tiles from a directory of files.
 */

class FileReader extends Reader {

    private File dir;		// parent directory of tile collection


/**
 *  Constructs a FileReader that will fetch tiles from the
 *  directory given by the URL.
 */
public FileReader(String url) throws Exception {
    if (url.startsWith("file:") || url.startsWith("FILE:")) {
	url = url.substring(5);
    }
    if (url.startsWith("//")) {
	url = url.substring(2);
    }
    dir = new File(new File(url).getParent());
    if (! dir.isDirectory()) {
	throw new FileNotFoundException(url);
    }
}



/**
 *  Loads a tile (synchronously) from a file.
 */
public void fetch(Tile t) throws Exception {
    t.req();
    File f = new File(dir, t.getName());
    InputStream in = new BufferedInputStream(new FileInputStream(f));
    t.resp();
    loadFile(in, t);
    in.close();
    t.done();
}



/**
 *  Returns a string representation of this reader.
 */
public String toString() {
    return "FileReader(" + dir.getPath() + ")";
}



} // class FileReader
