package topovista.tvsucker;

import java.io.*;

import topovista.grid.*;
import topovista.util.*;



/**
 *  Implements a utility for fetching (and discarding) tiles using TVTP.
 */
public class TVSucker {


static int concurrency;
static String spiral;
static String url;
static String infile;


/**
 *  Loads tiles and reports timings.
 *  See usage.
 */
public static void main(String[] args) throws Exception {

    String cmdname = "java TVSucker";
    Options o = new Options(new String[] {
	"-c n 1",	"fetch up to n tiles concurrently",
	"-r n",		"fetch n tiles randomly",
	"-s n",		"fetch n tiles in a spiral",
	"url",		"URL or path of first tile",
	"[listfile]",	"input file (list of tiles to fetch)",
    }, cmdname, args);

    try {
	concurrency = Integer.parseInt(o.getValue('c'));
	spiral = o.getValue('s');
	int i = o.getIndex();
	url = args[i];
	if (i + 1 < args.length) {
	    infile = args[i + 1];
	}
    } catch (Exception e) {
	o.usage(System.err, cmdname);
	System.exit(1);
    }

    // choose the reader based on the form of the URL
    final Reader r;
    if (url.startsWith("tvtp") || url.startsWith("TVTP")) {
	r = new TVTPReader(url);
    } else if (url.indexOf(':') > 1) {
	r = new HTTPReader(url);
    } else {
	r = new FileReader(url);
    }

    // choose the navigator based on command options
    String tstart = url.substring(url.lastIndexOf('/') + 1);
    final Navigator n;
    if (o.getValue('r') != null) {
	n = new RandomNav(tstart, Integer.parseInt(o.getValue('r')));
    } else if (o.getValue('s') != null) {
	n = new SpiralNav(tstart, Integer.parseInt(o.getValue('s')));
    } else if (infile != null) {
	n = new ListNav(new FileInputStream(infile));
    } else {
	n = new ListNav(System.in);
    }

    // load the tiles using N concurrent processes
    for (int i = 0; i < concurrency; i++) {
	new Thread() {
	    public void run() {
		Tile t;
		try {
		    while ((t = n.nextTile()) != null) {
			r.fetch(t);
			report(t);
		    }
		} catch (Throwable e) {
		    e.printStackTrace();
		    System.exit(1);
		}
	    }
	}.start();
    }

}



/**
 *  Outputs a standard report upon receipt of a tile.
 *  Called from main loop, and also from TVTPReader
 *  (after receiving an unsolicited tile).
 */
public static void report(Tile t) {
    announce(t.report());
}



/**
 *  Outputs a line, with synchronization to prevent confusion.
 */
public static void announce(String s) {
    synchronized (System.out) {
	System.out.println(s);
    }
}



} // class TVSucker
