package topovista.tvsucker;

/**
 *  A Timestamp memorializes an instant as an interval in milliseconds from
 *  the first Timestamp reference (or from a more recent base if reset).
 */

class Timestamp {

    public final long value;	// seconds and fraction since beginning


private static long base;	// the beginning of our "epoch"
static { reset(); }		// ensure initialization before first call



/**
 *  Constructs a Timestamp containing the interval since the last reset.
 */
public Timestamp() {
    value = System.currentTimeMillis() - base;
}



/**
 *  Returns the value of the timestamp, in milliseconds.
 */
public long getValue() {
    return value;
}



/**
 *  Returns the value in the form "t=nnn";
 */
public String toString() {
    return "t=" + value;
}



/**
 *  Resets the time base for subsequently created Timestamps.
 */
public static void reset() {
    base = System.currentTimeMillis();
}



} // class Timestamp
