package topovista.util;

import java.io.*;
import java.util.zip.*;



/**
 *  A GZInputStream reads an InputStream that may or may not be
 *  compressed in Gzip format, returning the uncompressed data.
 *
 *  @author Gregg Townsend, University of Arizona
 *  @version February, 2001
 */

public class GZInputStream extends FilterInputStream {



/**
 *  Constructs a GZInputStream from an underlying InputStream.
 */
public GZInputStream(InputStream in) throws IOException {
    super(new BufferedInputStream(in));
    this.in.mark(2);
    int b1 = this.in.read();
    int b2 = this.in.read();
    this.in.reset();
    if (256 * b2 + b1 == GZIPInputStream.GZIP_MAGIC) {
	this.in = new GZIPInputStream(this.in);
    }
}



/**
 *  Decompresses the files named as arguments, or standard input
 *  if no files are named.
 */
public static void main(String[] args) throws IOException {
    if (args.length > 0) {
	for (int i = 0; i < args.length; i++) {
	    zcat(new FileInputStream(args[i]));
	}
    } else {
	zcat(System.in);
    }
}

private static void zcat(InputStream in) throws IOException {
    int n;
    byte[] buf = new byte[1000];
    GZInputStream cis = new GZInputStream(in);
    while ((n = cis.read(buf, 0, buf.length)) >= 0) {
	System.out.write(buf, 0, n);
    }
    in.close();
}



} // class GZInputStream
